'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Observable = Observable;

var _debounce2 = require('./debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _delay2 = require('./delay');

var _delay3 = _interopRequireDefault(_delay2);

var _drop2 = require('./drop');

var _drop3 = _interopRequireDefault(_drop2);

var _filter2 = require('./filter');

var _filter3 = _interopRequireDefault(_filter2);

var _flatMapLatest2 = require('./flatMapLatest');

var _flatMapLatest3 = _interopRequireDefault(_flatMapLatest2);

var _fromEvent2 = require('./fromEvent');

var _fromEvent3 = _interopRequireDefault(_fromEvent2);

var _fromPromise2 = require('./fromPromise');

var _fromPromise3 = _interopRequireDefault(_fromPromise2);

var _interval2 = require('./interval');

var _interval3 = _interopRequireDefault(_interval2);

var _map2 = require('./map');

var _map3 = _interopRequireDefault(_map2);

var _merge2 = require('./merge');

var _merge3 = _interopRequireDefault(_merge2);

var _partition2 = require('./partition');

var _partition3 = _interopRequireDefault(_partition2);

var _pure2 = require('./pure');

var _pure3 = _interopRequireDefault(_pure2);

var _sliding2 = require('./sliding');

var _sliding3 = _interopRequireDefault(_sliding2);

var _throttle2 = require('./throttle');

var _throttle3 = _interopRequireDefault(_throttle2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Observable(activate) {

  function obs(val) {
    return arguments.length === 0 ? obs._lastValue === UNSET ? undefined : obs._lastValue : obs._add(val);
  }

  obs._subscribers = [];
  obs._activate = activate;
  obs._lastValue = UNSET;

  // Pre-bind _add as it's called as a detached function
  obs._add = function (val, name) {
    obs._lastValue = val;
    obs._parentName = name;

    pushNewValue(val, obs._subscribers, obs._name || name);

    return obs;
  };

  var protoKeys = Object.keys(proto);
  for (var i = 0; i < protoKeys.length; i++) {
    obs[protoKeys[i]] = proto[protoKeys[i]];
  }

  return obs;
}

var proto = {

  subscribe: function subscribe(cb) {
    var self = this;
    var _subscribers = this._subscribers,
        _add = this._add,
        _activate = this._activate,
        _name = this._name;


    if (_subscribers.length === 0) if (_activate) this._unsubscribe = _activate(_add);

    _subscribers.push(cb);

    if (this._lastValue !== UNSET) cb(this._lastValue, _name || this._parentName);

    return function _unsubscribe() {
      var index = _subscribers.indexOf(cb);

      if (index > -1) {
        _subscribers.splice(index, 1);

        if (_subscribers.length === 0) self._unsubscribe && self._unsubscribe();
      }
    };
  },

  named: function named(name) {
    this._name = name;
    return this;
  }

};

function pushNewValue(value, subscribers, name) {
  for (var i = 0; i < subscribers.length; i++) {
    subscribers[i](value, name);
  }
}

// Internal marker
var UNSET = {};

// Enrich the Observable "prototype" till (maybe) we have the |> operator!
proto.debounce = function (time) {
  return (0, _debounce3.default)(time, this);
};
proto.delay = function (time) {
  return (0, _delay3.default)(time, this);
};
proto.drop = function (count) {
  return (0, _drop3.default)(count, this);
};
proto.filter = function (fn) {
  return (0, _filter3.default)(fn, this);
};
proto.flatMapLatest = function (fn) {
  return (0, _flatMapLatest3.default)(fn, this);
};
proto.map = function (fn) {
  return (0, _map3.default)(fn, this);
};
proto.partition = function (predicate) {
  return (0, _partition3.default)(predicate, this);
};
proto.sliding = function (num) {
  return (0, _sliding3.default)(num, this);
};
proto.sliding2 = function () {
  return (0, _sliding2.sliding2)(this);
};
proto.throttle = function (time) {
  return (0, _throttle3.default)(time, this);
};

// Enrich the Observable object
Observable.pure = _pure3.default;
Observable.fromEvent = _fromEvent3.default;
Observable.fromPromise = _fromPromise3.default;
Observable.interval = _interval3.default;
Observable.merge = _merge3.default;